/* LICENSE:
Copyright (c) Members of the EGEE Collaboration. 2010.
See http://www.eu-egee.org/partners/ for details on the copyright
holders.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
implied.
See the License for the specific language governing permissions and
limitations under the License.

END LICENSE */

#ifndef _GLITE_CE_CREAM_CLIENT_API_CREAMEX_BASEEX_H__
#define _GLITE_CE_CREAM_CLIENT_API_CREAMEX_BASEEX_H__

#include<exception>
#include<string>
#include<ctime>
#include <cstring> // for memset
#include <boost/format.hpp>

namespace glite {
  namespace ce {
    namespace cream_client_api {
      namespace cream_exceptions {

	/// An object of this class is raised as an exception when the remote CREAM web service sends back a BaseFault to the client or when the fault type is not recognized
	class BaseException : public std::exception {
	  
	  std::string  m_methodname;
	  std::string  m_errorcode;
	  std::string  m_description;
	  std::string  m_faultcause;
	  time_t       m_timestamp;
	  
	public:

	  BaseException(const std::string& mn,
			const std::string& ec,
			const std::string& des,
			const std::string& fc,
			const time_t& t) 
	    : m_methodname(mn), m_errorcode(ec), m_description(des), m_faultcause(fc), m_timestamp(t) {}

	  virtual ~BaseException() throw() {}

	  virtual const char* what() const throw() { 
	    char timestamp[40]; 
	    struct tm TSTAMP;
	    memset((void*)timestamp, 0, 40); 
	    localtime_r(&m_timestamp, &TSTAMP);
	    strftime(timestamp, sizeof(timestamp), "%a %d %b %Y %T", &TSTAMP);

	    std::string whatinfo = boost::str( boost::format("MethodName=[%s] ErrorCode=[%s] Description=[%s] FaultCause=[%s] Timestamp=[%s]") % m_methodname % m_errorcode % m_description % m_faultcause % timestamp );

/*	    return (std::string("MethodName=[")+m_methodname
		    +"] ErrorCode=["+m_errorcode
		    +"] Description=["+m_description
		    +"] FaultCause=["+m_faultcause+"] Timestamp=["+std::string(timestamp)+"]");*/
	    return whatinfo.c_str();
	  }
	  virtual const char* getMethodName() const { return m_methodname.c_str(); }
	  virtual const char* getErrorCode() const { return m_errorcode.c_str(); }
	  virtual const char* getDescription() const { return m_description.c_str(); }
	  virtual const char* getFaultCause() const { return m_faultcause.c_str(); }
	  virtual time_t      getTimeStamp() const { return m_timestamp; }
	};
      }
    }
  }
}

#endif
